/*
 * bar graph v 0.6
 * created August 2011
 * for the Canada Food Guide Visualizer project
 * GSOC 2011
 * author Caroline Ross
 */

// declare instance of bar graph object
BarGraph recommendedDailyIntake;


//declare each color
//green
color greenGroup = color(0, 146, 69);
//yellow
color yellowGroup = color(251, 176, 59);
//blue
color blueGroup = color(63, 169, 245);
//red
color redGroup = color(193, 39, 45);

//create an array of the colours
color[] groupColors = {greenGroup, yellowGroup, blueGroup, redGroup};

// PFont is the font class for processing
PFont plotFont;


// set up function runs once
void setup() {
  size(600, 600);
  smooth();
  noStroke();
  rectMode(CORNERS);

  // create instance of BarGraph class for the recommended daily food intake
  recommendedDailyIntake = new BarGraph (width/7, 500);

  // make font
  plotFont = createFont("SansSerif", 12);
  textFont(plotFont);
}

// draw function runs continuously
void draw() {
  // draw background
  background(255);
  
  // draw instance of BarGraph class
  recommendedDailyIntake.draw();
}


// CLASSES

// create pie chart class
class BarGraph {

  // declare instance variables: x position and y position and FoodGroup class components
  private int graphX, graphY;
  
   // declare instance variables: x position, bar height, colour and title name
  private int xPos;
  private int xPosDistance =   120;
  private float barHeight;
  private int heightMultiplier = 45;
  private color barColor;
  private String barTitle;
  private int loopCount = 0;
  private int barWidth = width/10;
  private float yStart = width*0.8;
  // create variable that represents the length of the array of food servings (4)
  private int servingsLength = visualizer.data.foodGroupData[userInputData].servings.length;
  
  // variable to represent data choice selection of DataType class
  // int userInputData;
  
  // CONSTRUCTOR - class function to create new objects with the properties given
  public BarGraph(int barX, int barY) {
    graphX = barX;
    graphY = barY;
  }

  // SETTERS
  // function sets x coordinate
  public void setX (int x) {
    graphX = x;
  }

  // function sets y coordinate
  public void setY (int y) {
    graphY = y;
  }

  // GETTERS
  // function gets x coordinate value
  public int getX() {
    return graphX;
  }

  // function gets x coordinate value
  public int getY() {
    return graphY;
  }

  // BEHAVIOR of BarGraph class
  public void draw() {
    
    // set initial x position
    xPos = graphX;
    
    // for each column in the number of columns
    for (int i = 0; i < servingsLength; i++) {
          
      int serving = visualizer.data.foodGroupData[userInputData].servings[i];
      //println("serving "+serving);
      
      // track number of times for loop runs
      loopCount++; 
      // add current number to the total number of servings until max
      if (loopCount < servingsLength) {
        xPos+=xPosDistance;
      }  else {
        loopCount = 0;
        xPos=graphX;
      }
      
      //println("loopCount "+loopCount);
      //println("xPos "+xPos);

      
      barHeight = serving*heightMultiplier;
      //println("barHeight "+barHeight);
      
      fill(groupColors[i], 180);
      stroke(groupColors[i]);
      strokeWeight(2);

      //draw bar
      rect(xPos, yStart, xPos+barWidth, yStart-barHeight);

      // draw title of food group
      fill(groupColors[i]);
      textSize(12);
      noStroke();
      text(visualizer.data.titleList.shortTitles[i], xPos, yStart+30);
      
    }
  }

  // closing BarGraph class bracket
}


