/*
 * Food Guide Visualizer Application 1.0
 * created August 2011
 * for the Canada Food Guide Visualizer project
 * GSOC 2011
 * author Caroline Ross
 */

var userInputData = 8;

// the global namespace
var visualizer = visualizer || {};
(function($, fluid) {
	fluid.defaults("visualizer.visualizer", {
		selectors : {
			age : ".age",
			genderrow : ".genderRow",
			genderlabel : ".genderLabel",
			genderbutton : ".genderButton",
			dataoutput : ".dataDisplay",
			thumbnailrow : ".thumbLi",
			thumbnailimage : ".thumbnailImage",
			canvascontainer : ".visual-container",
			canvaselement : "#canvas-visuals"
		},
		selectorsToIgnore : ["dataoutput", "canvascontainer", "canvaselement"],
		/*the selector to be repeated*/
		repeatingSelectors : ["genderrow", "thumbnailrow"],
		produceTree : "visualizer.produceTree"
	});

	// function to refresh the canvas element
	var refreshCanvasElement = function(that) {
		// create a copy of the canvas element
		var newCanvas = that.locate("canvaselement").clone();
		// remove old canvas from the dom
		that.locate("canvaselement").remove();
		// convert string from data model to integer
		var newIndex = parseInt(that.model.formList.selection);
		// set new processing sketch in canvas clone - now the canvas instance in the dom
		newCanvas.attr("data-processing-sources", that.model.formList.choices[newIndex].pde);
		// insert this copy of the canvas element into dom in the container that holds the canvas element
		that.locate("canvascontainer").append(newCanvas);
		// explicit instructions to load the processing sketch from the .pde file to the canvas element
		Processing.loadSketchFromSources(newCanvas[0],[that.model.formList.choices[newIndex].pde]);
	};
	
	// function to refresh data display
	var refreshDataDisplay = function(that) {
		var dataString = "";
		// function to create strings of title and food servings for each food serving
		for (var i = 0; i < visualizer.data.titleList.longTitles.length; i++) {
			dataString += visualizer.data.titleList.longTitles[i] + ": " + visualizer.data.foodGroupData[userInputData].servings[i] + "<br />";
		}
		// locate the html element where the data will display
		that.locate("dataoutput").empty().append(dataString);
	};
	
	// my component public function (creator function)
	visualizer.visualizer = function(container, options) {
		var that = fluid.initRendererComponent("visualizer.visualizer", container, options);
		that.produceTree = visualizer.produceTree;
		// function that runs any time the model changes
		that.applier.modelChanged.addListener("*", function(newModel) {
			var newDataRow = parseInt(newModel.ageList.selection) + parseInt(newModel.genderList.selection);
			userInputData = newDataRow;
			refreshCanvasElement(that);
			refreshDataDisplay(that);
		});
		refreshDataDisplay(that);
		return that;
	};
	
	// function that handles user clicks on the menu images
	makeFormClickHandler = function(that) {
		return function() {
			// extract substring out of string
			var visArrayPos = this.attributes.id.nodeValue.substring(13,14);
			// since the value of 0 is missing for the first position in the array, replace the value with 0
			if(visArrayPos === ":") {
				visArrayPos = "0";
			}
			// updating the model
			that.applier.requestChange("formList.selection", visArrayPos);
			refreshCanvasElement(that);
		}
	};
	visualizer.produceTree = function(that) {
		var tree = {
			age : {
				selection : "${ageList.selection}",
				optionlist : "${ageList.codes}",
				optionnames : "${ageList.names}"
			},
			/* expander generates radio button entries with options*/
			expander : [{
				type : "fluid.renderer.selection.inputs",
				rowID : "genderrow",
				labelID : "genderlabel",
				inputID : "genderbutton",
				selectID : "genderRadio",
				tree : {
					selection : "${genderList.selection}",
					optionlist : "${genderList.codes}",
					optionnames : "${genderList.names}"
				}
			},
			/* 2nd expander generates menu thumbnail button entries with options*/
			{
				type : "fluid.renderer.repeat",
				repeatID : "thumbnailrow",
				controlledBy : "formList.choices",
				pathAs : "thumbs",
				tree : {
					thumbnailimage : {
						target : "${{thumbs}.thumb}",
						decorators : {
							type : "jQuery",
							func : "click",
							args : makeFormClickHandler(that)
						}
					}
				}
			}]
		};
		return tree;
	};
	visualizer.initVisualization = function(container) {
		var applier = fluid.makeChangeApplier(visualizer.data);
		var myVisualizer = visualizer.visualizer(container, {
			model : visualizer.data,
			//rendererOptions : {
				//debugMode : true
			//}
		});
		myVisualizer.refreshView();
	};
// when updating fluid infusion versions the name below is the only instance that requires updating
})(jQuery, fluid_1_4);
