/*
 * network graph v 0.9.5
 * created August 2011
 * for the Canada Food Guide Visualizer project
 * GSOC 2011
 * author Caroline Ross
 */


// declare instance of network graph object
NetworkGraph recommendedDailyIntake;

// declare x and y position variables
int xPos, yPos;

// declare instances of Hub class, the hubs for each food group
Hub greenHub;
Hub yellowHub;
Hub blueHub;
Hub redHub;

// declare hub x and y coordinates
float hubX, hubY;
//declare serving x and y coordinates
float servingX, servingY;

// declare instances of Serving class (network nodes), representing each food group serving as a circle object
Serving greenServing;
Serving yellowServing;
Serving blueServing;
Serving redServing;

// set initial Serving and Hub object force variables
float gravity = 0.001;
float mass = 1.0;

//declare each color
//red
color redGroup = color(193, 39, 45);
//blue
color blueGroup = color(63, 169, 245);
//yellow
color yellowGroup = color(251, 176, 59);  
//green
color greenGroup = color(0, 146, 69);

//create an array of the colours
color[] groupColors = {greenGroup, yellowGroup, blueGroup, redGroup};

// PFont is the font class for processing
PFont plotFont;


// set up function runs once 
void setup() {
  size(600, 600);
  smooth();
  
  // create instance of NetworkGraph class for the recommended daily food intake
  recommendedDailyIntake = new NetworkGraph ((width), (height));

  // make font
  plotFont = createFont("SansSerif", 12);
  textFont(plotFont);

  // create instances of food group hubs - Hub class, input; x, y, mass, gravity, color, and title
  greenHub = new Hub(hubX, hubY, mass, gravity, groupColors[0], visualizer.data.titleList.shortTitles[0]);
  yellowHub = new Hub(hubX, hubY, mass, gravity, groupColors[1], visualizer.data.titleList.shortTitles[1]);
  blueHub = new Hub(hubX, hubY, mass, gravity, groupColors[2], visualizer.data.titleList.shortTitles[2]); 
  redHub = new Hub(hubX, hubY, mass, gravity, groupColors[3], visualizer.data.titleList.shortTitles[3]);

  // create instances of food group servings - Serving class, input; x, y mass, gravity, color
  greenServing = new Serving(visualizer.data.foodGroupData[userInputData].servings[0], servingX, servingY, mass, gravity, groupColors[0]);
  yellowServing = new Serving(visualizer.data.foodGroupData[userInputData].servings[1], servingX, servingY, mass, gravity, groupColors[1]);
  blueServing = new Serving(visualizer.data.foodGroupData[userInputData].servings[2], servingX, servingY, mass, gravity, groupColors[2]);
  redServing = new Serving(visualizer.data.foodGroupData[userInputData].servings[3], servingX, servingY, mass, gravity, groupColors[3]);
}


// draw function runs continuously
void draw() {
  background (255);

  // draw instance of network graph class
  recommendedDailyIntake.draw();  
}


// CLASSES

// create Hub class representing the hub circle of each food group object
class Hub {
  // declare instance variables for hub objects
  // x and y coordinates of hub
  private float hubX, hubY;
  // x and y axis velocities
  private float hubVelocityX, hubVelocityY;
  private float hubGravity;
  private float hubMass;
  private float stiffness = 0.2;
  private float damping = 0.3;
  private color foodGroupColor;
  private String foodGroupTitle;
  private float hubRadius;
  // length of distance from hub to hub
  private float hubDistance;
  // create variable that represents the length of the array of food servings (4)
  private int servingsLength = visualizer.data.foodGroupData[userInputData].servings.length;
  // add title x position offset
  // add hub colour group position offset
  

  // CONSTRUCTOR - class function to create a shape object with the properties given
  public Hub (float hx, float hy, float mass, float gravity, color colour, String title) {
    hubX = hx;
    hubY = hy;
    hubMass = mass;
    hubGravity = gravity;
    foodGroupColor = colour;
    hubRadius = width/14;
    hubDistance = hubRadius*6;
    foodGroupTitle = title;
    // add title x position offset
    // add hub colour group position offset
  }

  // SETTERS create functions for each shape attribute (class variables), validity checks

  // function sets x position
  public void setHubXPosition (float hx) {
    hubX = hx;
  }

  // function sets y position
  public void setHubYPosition (float hy) {
    hubY = hy;
  }
  
   // function sets object colour
  public void setFoodGroupColor (color colour) {
    foodGroupColor = colour;
  }
  
  // function sets the food group title
  public void setFoodGroupTitle (String title) {
    foodGroupTitle = title;
  }

  // GETTERS return values of object properties for instance variables
  
  // get x position
  public float getHubXPosition () {
    return hubX;
  }

  //  get y position
  public float getHubYPosition () {
    return hubY;
  }
  
  // get food group color
  public color getFoodGroupColor () {
    return foodGroupColor;
  }
  
    
   // get food group color
  public String getFoodGroupTitle () {
    return foodGroupTitle;
  }

  // BEHAVIOR of Hub class
  public void draw (float hubTargetX, float hubTargetY) {
    
    // the next 10 lines of code are inspired by the Traer.Physics Processing library http://murderandcreate.com/physics/
    //for X coordinates
    float forceX = (hubTargetX - hubX)*stiffness;
    float accelerateX = forceX/hubMass;
    hubVelocityX = damping*(hubVelocityX + accelerateX);
    hubX += hubVelocityX;
    
    // for Y coordinates
    float forceY = (hubTargetY- hubY) * stiffness;
    forceY += hubGravity;
    float accelerateY = forceY/hubMass;
    hubVelocityY = damping * (hubVelocityY + accelerateY);
    hubY += hubVelocityY;
    
    // assign color
    fill(255);
    stroke(foodGroupColor, 180);
    
    // draw hub
    ellipse (hubX, hubY, hubRadius*2, hubRadius*2);
    
    // draw titles of food groups
    fill(foodGroupColor);
    noStroke();
    textSize(12);
    //for(int i=0; i< servingsLength; i++) {
      //foodGroupTitle = visualizer.data.titleList.shortTitles[i];
      text(foodGroupTitle, hubX-(hubRadius/3), hubY);
      
    //constrain position to within display boundaries
    hubX = constrain(hubX, 0+(hubRadius*3), width-(hubRadius*3));
    hubY = constrain(hubY, 0+(hubRadius*3), height-(hubRadius*3));
  }

  // Hub class closing bracket
}


// create Serving class representing each ball object
class Serving {
  // declare variable for number of food servings
  private int foodServings;
  // declare instance variables
  private float servingX, servingY;
  // x and y axis velocities
  private float velocityX, velocityY;
  private float gravity;
  private float mass;
  private float stiffness = 0.2;
  private float damping = 0.3;
  private color foodGroupColor;
  private float radius;
  // angle of branch offshoot from hub
  float theta;
  // length of line from hub to node
  float lineLength;
  float sumServingsArray = 0;
  float totalDegrees = 360;

  // CONSTRUCTOR - class function to create a shape object with the properties given
  public Serving (int servings, float x, float y, float m, float g, color colour) {
    foodServings = servings;
    servingX = x;
    servingY = y;
    mass = m;
    gravity = g;
    foodGroupColor = colour;
    radius = width/24;
    lineLength = radius*3.6;
  }

  // SETTERS create functions for each shape attribute (class variables), validity checks
  // function sets number of food servings in food group
  public void setFoodServings (int servings) {
    foodServings = servings;
  }

  // function sets x position
  public void setServingXPosition (float x) {
    servingX = x;
  }

  // function sets y position
  public void setServingYPosition (float y) {
    servingY = y;
  }
  
   // function sets food group colour
  public void setFoodGroupColor (color colour) {
    foodGroupColor = colour;
  }

  // GETTERS return values of object properties for instance variables
  // get number of food servings in food group
  public int getFoodServings () {
    return foodServings;
  }
  
  // get x position
  public float getServingXPosition () {
    return servingX;
  }

  //  get y position
  public float getServingYPosition () {
    return servingY;
  }
  
  // get food group color
  public color getFoodGroupColor () {
    return foodGroupColor;
  }

  // BEHAVIOR of Serving class
  public void draw (float hubXPos, float hubYPos) {
    /*
    // x coordinate movement
    float forceX = (hubXPos - servingX) * stiffness;
    float accelerateX = forceX / mass;
    velocityX = damping*(velocityX + accelerateX);
    servingX += velocityX;
    
    // y coordinate movement
    float forceY = (hubYPos - servingY) * stiffness;
    forceY += gravity;
    float accelerateY = forceY / mass;
    velocityY = damping * (velocityY + accelerateY);
    servingY += velocityY;
     */
      
      // the angle of increase is the serving percentage of a circle
      theta = (totalDegrees/foodServings);
      
      // assign color of node
      fill(foodGroupColor, 180);
      stroke(foodGroupColor); 
       
      // function to draw number of servings in food groups
      for (int i = 0; i < foodServings; i++) {
        
          // draw new serving node at an angle and a distance specified from the hub
          servingX = hubXPos + cos(radians(theta))*lineLength;
          servingY = hubYPos + sin(radians(theta))*lineLength;
      
          // draw node connections
          ellipse (servingX, servingY, radius/4, radius/4);
          line (servingX, servingY, hubXPos, hubYPos);
          // draw node
          ellipse (servingX, servingY, radius*2, radius*2);
          
          // change position of node
          theta += (totalDegrees/foodServings);
    
      }
    
      
    //constrain position to within display boundaries
    servingX = constrain(servingX, 0+(radius*2), width-(radius*2));
    servingY = constrain(servingY, 0+(radius*2), height-(radius*2));
  }

// Serving class closing bracket
}


// create network graph class
class NetworkGraph {

  // declare instance variables: x position and y position and FoodGroup class components
  private float graphX, graphY;
  private Hub hub;
  private Serving serving;

  // CONSTRUCTOR - class function to create new network graph objects 
  public NetworkGraph(float x, float y) {
    graphX = x;
    graphY = y;
  }

  // SETTERS
  // function sets x coordinate
  public void setX (float x) {
    graphX = x;
  }

  // function sets y coordinate
  public void setY (float y) {
    graphY = y;
  }

  // GETTERS
  // function gets x coordinate value
  public float getX() {
    return graphX;
  }

  // function gets x coordinate value
  public float getY() {
    return graphY;
  }

  // BEHAVIOR of NetworkGraph class
  public void draw() {
    
    // draw instance of Serving class using food group hub coordinates
    greenServing.draw(greenHub.hubX, greenHub.hubY);
    // draw instance of Hub class using mouse x and y coordinates
    greenHub.draw(mouseX, mouseY);
   
    yellowServing.draw(yellowHub.hubX, yellowHub.hubY);
    yellowHub.draw(greenHub.hubX+270, greenHub.hubY+260);
    
    redServing.draw(redHub.hubX, redHub.hubY);
    redHub.draw(greenHub.hubX+260, greenHub.hubY+10);
    
    blueServing.draw(blueHub.hubX, blueHub.hubY);
    blueHub.draw(greenHub.hubX, greenHub.hubY+270);

  }

  // closing NetworkGraph class bracket
}