/*
 * pie chart v 1.0
 * created August 2011
 * for the Canada Food Guide Visualizer project
 * GSOC 2011
 * author Caroline Ross
 */


// declare instance of pie class object
Pie recommendedDailyIntake;

// declare variables for pie x and y coordinates
int pieX, pieY;

// declare instances of Title class objects
Title vegetablesTitle;
Title grainsTitle;
Title milkTitle;
Title meatTitle;

//declare each color
//green
color greenGroup = color(0, 146, 69);
//yellow
color yellowGroup = color(251, 176, 59);
//blue
color blueGroup = color(63, 169, 245);
//red
color redGroup = color(193, 39, 45);

//create an array of the colours
color[] groupColors = {greenGroup, yellowGroup, blueGroup, redGroup};

// PFont is the font class for processing
PFont plotFont;

// set up function runs once
void setup() {
  size(600, 600);
  smooth(); 

  // create instance of Pie class for recommended daily food intake
  recommendedDailyIntake = new Pie (width/2, height/2);

  // make font
  plotFont = createFont("SansSerif", 12);
  textFont(plotFont);

  // create instances of Title class
  vegetablesTitle = new Title(visualizer.data.titleList.shortTitles[0], 40, 50, groupColors[0]);
  grainsTitle = new Title(visualizer.data.titleList.shortTitles[1], 40, 70, groupColors[1]);
  meatTitle = new Title(visualizer.data.titleList.shortTitles[2], 40, 90, groupColors[2]);
  milkTitle = new Title(visualizer.data.titleList.shortTitles[3], 40, 110, groupColors[3]);
}



// draw function runs continuously
void draw() {
  background(255);

  // draw instance of pie class
  recommendedDailyIntake.draw(pieX, pieY);
}

// CLASSES

// create class for titles of food groups
class Title {

  // declare instance variables: x position, y position and title name
  private String foodGroupTitle;
  private float titleXPos;
  private float titleYPos;
  private color foodGroupColor;

  // CONSTRUCTOR - class function to create a shape object with the properties given
  public Title (String title, float titleX, float titleY, color colour) {
    foodGroupTitle = title;
    titleXPos = titleX;
    titleYPos = titleY;
    foodGroupColor = colour;
  }

  // SETTERS create functions for each shape attribute (class variables), validity checks

  // function sets the food group title
  public void setSliceTitle (String title) {
    foodGroupTitle = title;
  }

  // function sets food group title x position
  public void setTitleXPosition (float titleX) {
    titleXPos = titleX;
  }

  // function sets food group title y position
  public void setTitleYPosition (float titleY) {
    titleYPos = titleY;
  }

  // function sets object colour
  public void setSliceColor (color colour) {
    foodGroupColor = colour;
  }

  // GETTERS return values of object properties for instance variables

  // get food group title
  public String getSliceTitle () {
    return foodGroupTitle;
  }

  // get title x position
  public float getTitleXPosition () {
    return titleXPos;
  }

  //  get title y position
  public float getTitleYPosition () {
    return titleYPos;
  }

  // get food group color
  public color getSliceColor () {
    return foodGroupColor;
  }

  // BEHAVIOR of Title class
  public void draw () {
    // draw titles of food groups
    fill(foodGroupColor);
    noStroke();
    textSize(12);
    text(foodGroupTitle, titleXPos, titleYPos);
  }
}



// create pie chart class
class Pie {

  // declare instance variables: x position and y position and pie chart slice class
  private int pieX, pieY;
  // new x and y coordinates
  private int nX, nY;

  // declare pie diameter
  private float diameter;
  private int foodServings;
  
  private float totalRadians;
  // angle of each food group, slice width
  private float theta;
  private float angleStart = radians(0);
  private float angleEnd;
  private int totalServings;
  private int loopCount = 0;
  // create variable that represents the length of the array of food servings (4)
  private int servingsLength = visualizer.data.foodGroupData[userInputData].servings.length;

  // CONSTRUCTOR - class function to create new pie chart objects 
  public Pie(int x, int y) {
    pieX = x;
    pieY = y;
    // set variable for pie diameter
    diameter = min(width, height) * 0.8;
  }

  // SETTERS

  // function sets x coordinate
  public void setX (int x) {
    pieX = x;
  }

  // function sets y coordinate
  public void setY (int y) {
    pieY = y;
  }

  // GETTERS

  // function gets x coordinate value
  public int getX() {
    return pieX;
  }

  // function gets x coordinate value
  public int getY() {
    return pieY;
  }

  // BEHAVIOR of pie class
  public void draw(float TargetX, float TargetY) {
    
    // total radians is equal to 2pi, approximately 6.2831855 or 360 degrees
    totalRadians = 6.288;
	    
    // for each serving in the number of servings array (4)
    for (int i = 0; i < servingsLength; i++) {
      // userInputData is supplied by the global namespace
      foodServings = visualizer.data.foodGroupData[userInputData].servings[i];
      
      // track number of times for loop runs
      loopCount++; 
      // add current number to the total number of servings until max
      if (loopCount <= servingsLength) {
        totalServings += foodServings;
      }  else {
        loopCount = servingsLength;
      }
      
      // calculate the percentage to divide by to get quantity of radians
      float percentageDivider = (totalServings/totalRadians);
      //println("percentageDivider "+percentageDivider);
      
      // calculcate the percentage of the angle in radians
      theta = foodServings/percentageDivider;
      //theta = (foodServings/totalServings)*totalRadians;
      //println("theta "+theta);
     
      // set angle end point
      float angleEnd = angleStart + theta;  
      
      // set pie slice colour
      fill(groupColors[i], 180);
      strokeWeight(4);
      stroke(groupColors[i]);

      //draw food group pie slice
      arc(recommendedDailyIntake.pieX, recommendedDailyIntake.pieY, diameter, diameter, angleStart, angleEnd);
      // change position of starting angle 
      angleStart += theta;
      
      // reset angle when over 360
      if (angleStart>=totalRadians) {
        angleStart=radians(0);
    }
    }

    // draw instances of food group Title class
    vegetablesTitle.draw();
    grainsTitle.draw();
    meatTitle.draw();
    milkTitle.draw();

    // declare variable to keep pie in display boundaries (should be pie radius) 
    int pieConstraint = 288;

    //constrain position to within display boundaries
    pieX = constrain(pieX, pieConstraint, width-pieConstraint);
    pieY = constrain(pieY, pieConstraint, height-pieConstraint);
  }


  // closing pie class bracket
}